import { Message } from 'eris';
import { createPaginationEmbed } from 'eris-pagination';
import { Client, Command, RichEmbed } from '../class';

export default class extends Command {
  constructor(client: Client) {
    super(client);
    this.name = 'members';
    this.description = 'Gets a list of members in the server or members in a specific role.';
    this.usage = `${this.client.config.prefix}members [role name]`;
    this.guildOnly = true;
    this.enabled = false;
  }

  public async run(message: Message, args: string[]) {
    try {
      await this.mainGuild.fetchAllMembers();
      if (!args[0]) {
        const embed = new RichEmbed();
        const membersOnline = this.mainGuild.members.filter((member) => member.status === 'online');
        const membersIdle = this.mainGuild.members.filter((member) => member.status === 'idle');
        const membersDnd = this.mainGuild.members.filter((member) => member.status === 'dnd');
        const membersOffline = this.mainGuild.members.filter((member) => member.status === 'offline');
        const membersBots = this.mainGuild.members.filter((member) => member.user.bot === true);
        const membersHuman = this.mainGuild.members.filter((member) => member.user.bot === false);

        embed.setTitle('Members');
        embed.addField('Total', `${this.mainGuild.members.size}`, true);
        embed.addField('Humans', `${membersHuman.length}`, true);
        embed.addField('Bots', `${membersBots.length}`, true);
        embed.addBlankField();
        embed.addField('Online', `${membersOnline.length}`, true);
        embed.addField('Idle', `${membersIdle.length}`, true);
        embed.addField('Do Not Disturb', `${membersDnd.length}`, true);
        embed.addField('Offline', `${membersOffline.length}`, true);
        embed.setFooter(this.client.user.username, this.client.user.avatarURL);
        embed.setTimestamp();

        return message.channel.createMessage({ embed });
      }

      const role = this.client.util.resolveRole(args.join(' '), this.mainGuild);
      if (!role) return this.error(message.channel, 'The role you specified doesn\'t exist.');
      const membersArray: [{name: string, value: string}?] = [];
      const membersOnline: string[] = [];
      const membersIdle: string[] = [];
      const membersDnd: string[] = [];
      const membersOffline: string[] = [];
      for (const member of this.mainGuild.members.filter((m) => m.roles.includes(role.id)).sort((a, b) => a.username.localeCompare(b.username))) {
        switch (member.status) {
          case 'online':
            membersOnline.push(`${member.user.username}#${member.user.discriminator} | <@${member.user.id}>`);
            break;
          case 'idle':
            membersIdle.push(`${member.user.username}#${member.user.discriminator} | <@${member.user.id}>`);
            break;
          case 'dnd':
            membersDnd.push(`${member.user.username}#${member.user.discriminator} | <@${member.user.id}>`);
            break;
          case 'offline':
            membersOffline.push(`${member.user.username}#${member.user.discriminator} | <@${member.user.id}>`);
            break;
          case undefined:
            membersOffline.push(`${member.user.username}#${member.user.discriminator} | <@${member.user.id}>`);
            break;
          default:
            break;
        }
      }
      if (membersOnline.length > 0) membersArray.push({ name: 'Online', value: membersOnline.join('\n') });
      if (membersIdle.length > 0) membersArray.push({ name: 'Idle', value: membersIdle.join('\n') });
      if (membersDnd.length > 0) membersArray.push({ name: 'Do Not Disturb', value: membersDnd.join('\n') });
      if (membersOffline.length > 0) membersArray.push({ name: 'Offline', value: membersOffline.join('\n') });
      const membersSplit = this.client.util.splitFields(membersArray);
      const cmdPages: RichEmbed[] = [];
      membersSplit.forEach((split) => {
        const embed = new RichEmbed();
        embed.setTitle(`Members in ${role.name}`);
        embed.setDescription(`Members in Role: ${membersOnline.length + membersIdle.length + membersDnd.length + membersOffline.length}`);
        embed.setColor(role.color);
        embed.setFooter(this.client.user.username, this.client.user.avatarURL);
        embed.setTimestamp();
        split.forEach((c) => embed.addField(c.name, c.value));
        return cmdPages.push(embed);
      });
      if (cmdPages.length === 1) return message.channel.createMessage({ embed: cmdPages[0] });
      return createPaginationEmbed(message, cmdPages);
    } catch (err) {
      return this.client.util.handleError(err, message, this);
    }
  }
}
